<?php
/*
 * diag_edit.php
 *
 * 用于在OPNsense上编辑文件，并支持浏览文件功能。
 */

require_once("guiconfig.inc");
include("head.inc");
include("fbegin.inc");

// 设置允许访问的根目录
$baseDir = "/";  // 根据需求限制为特定目录

// 获取请求参数
$currentDir = realpath(isset($_GET['dir']) ? $_GET['dir'] : $baseDir);
$filePath = isset($_GET['file']) ? realpath($_GET['file']) : '';

// 验证当前路径是否在允许的目录范围内
$baseDirReal = realpath($baseDir);
if (!$currentDir || strpos($currentDir, $baseDirReal) !== 0) {
    die("不允许访问该目录。");
}

// 验证文件路径是否在允许范围内
if ($filePath && strpos($filePath, $baseDirReal) !== 0) {
    die("文件路径非法或超出范围。");
}

// 保存文件内容
if ($_SERVER["REQUEST_METHOD"] === "POST" && isset($_POST['content'], $_POST['file'])) {
    $content = $_POST['content'];
    $file = realpath($_POST['file']);

    if ($file && strpos($file, $baseDirReal) === 0 && is_writable($file)) {
        file_put_contents($file, $content);
        $message = "<div class='alert alert-success'>保存成功！</div>";
        // 保存后，跳转到文件所在目录
        $currentDir = dirname($file);
    } else {
        $message = "<div class='alert alert-danger'>无法保存文件，请检查权限或路径。</div>";
    }
} else {
    $message = "";
}

// 加载文件内容
$content = ($filePath && file_exists($filePath)) ? file_get_contents($filePath) : '';

// 获取目录中的文件和子目录
$files = [];
if (is_dir($currentDir) && $handle = opendir($currentDir)) {
    while (false !== ($entry = readdir($handle))) {
        if ($entry !== "." && $entry !== "..") {
            $fullPath = $currentDir . DIRECTORY_SEPARATOR . $entry;
            $files[] = [
                'name' => $entry,
                'type' => is_dir($fullPath) ? 'dir' : 'file',
                'path' => $fullPath
            ];
        }
    }
    closedir($handle);

    // 排序：目录优先，然后按名称排序
    usort($files, function ($a, $b) {
        if ($a['type'] === $b['type']) {
            return strcasecmp($a['name'], $b['name']);
        }
        return $a['type'] === 'dir' ? -1 : 1;
    });
}

// 获取上一级目录
$parentDir = realpath($currentDir . DIRECTORY_SEPARATOR . '..');
if (strpos($parentDir, $baseDirReal) !== 0) {
    $parentDir = null; // 超出允许范围，不显示上一级
}

// HTML 输出
?>

<section class="page-content-main">
    <div class="container-fluid">
        <div class="row">
            <!-- 编辑区 -->
            <section class="col-xs-12">
                <div class="content-box tab-content table-responsive __mb">
                    <table class="table table-striped">
                        <tbody>
                            <tr>
                                <td>
                                    <strong>编辑文件：</strong>
                                </td>
                            </tr>
                            <tr>
                                <td>
                                    <div class="edit">
                                        <?php if ($filePath): ?>
                                            <?php echo htmlspecialchars($filePath); ?>
                                    </div>
                                </td>
                            </tr>
                            <tr>
                                <td>
                                    <form action="" method="post">
                                        <input type="hidden" name="file"
                                            value="<?php echo htmlspecialchars($filePath); ?>">
                                        <textarea style="max-width: none;overflow-x: scroll;word-wrap:break-word;" rows="30" name="content"><?php echo htmlspecialchars($content); ?></textarea>
                                </td>
                            </tr>
                            <tr>
                                <td>
                                    <button type="submit" class="btn btn-primary">
                                        <i class="fa fa-save"></i> 保存
                                    </button>
                                    </form>
                                <?php else: ?>
                                    注意：该功能仅限高级用户使用！如果你不知道要做什么，请不要使用该功能。
                                <?php endif; ?>
                                </td>
                            </tr>
                        </tbody>
                    </table>
                </div>
            </section>

            <!-- 浏览区 -->
            <section class="col-xs-12">
                <div class="content-box tab-content table-responsive __mb">
                    <form action="" method="get" class="mb-3">
                        <div class="form-group">
                            <input type="text" id="load-file" name="file" class="form-control" placeholder="输入文件路径">
                        </div>
                        <button type="submit" class="btn btn-secondary">
                            <i class="fa fa-folder-open"></i> 加载文件
                        </button>
                    </form>
                    <table class="table table-striped">
                        <tbody>
                            <tr>
                                <td>
                                    <strong>当前目录:</strong> <?= htmlspecialchars($currentDir); ?>
                                </td>
                            </tr>
                            <?php if ($parentDir): ?>
                                <tr>
                                    <td>
                                        <a href="?dir=<?= urlencode($parentDir); ?>">
                                            <i class="fa fa-arrow-up"></i> 返回上一级
                                        </a>
                                    </td>
                                </tr>
                            <?php endif; ?>
                            <tr>
                                <td>
                                    <ul style="list-style: none; padding-left: 0; display: flex; flex-wrap: wrap;">
                                        <?php foreach ($files as $file): ?>
                                            <li style="width: 30%; margin-bottom: 10px;">
                                                <?php if ($file['type'] === 'dir'): ?>
                                                    <a href="?dir=<?= urlencode($file['path']); ?>">
                                                        <i class="fa fa-folder"></i> <?= htmlspecialchars($file['name']); ?>
                                                    </a>
                                                <?php else: ?>
                                                    <a href="?file=<?= urlencode($file['path']); ?>&dir=<?= urlencode($currentDir); ?>">
                                                        <i class="fa fa-file"></i> <?= htmlspecialchars($file['name']); ?>
                                                    </a>
                                                <?php endif; ?>
                                            </li>
                                        <?php endforeach; ?>
                                    </ul>
                                </td>
                            </tr>
                        </tbody>
                    </table>
                </div>
            </section>
        </div>
    </div>
</section>

<?php include("foot.inc"); ?>